;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;/         
;/ Filename: JTC_NTSC_240_192_v00x.s
;/
;/ Author: JT Cook - http://www.avalondreams.com
;/ 
;/ Description: 240x192 tile/sprite driver (32x32 scrolling tile map - 30x24 visible)
;/ Supports 8 16x16 sprites and 8-way scrolling, split screen
;/ Does not use palette, only reads colors from tiles and sprites
;/ Todo:
;/ - Fix timing error (first few scanlines "bend" to the right)
;/ - Timing error when X position starts with tile 31 (last time) as first tile?
;/ *** NOTE: Added screen division, need to add tilemap to screen division
;/ Fixed:
;/ - W7 register wasn't pushed/poppped
;/ ---
;/ - now erases sprite pixels for last tile
;/ ---
;/ - added map scrolling
;/ - extra NOP in Pixel 1 in Tile drawing algo
;/ - pushed and popped all the W registers I am using so it doesn't interfere with the C code
;/ - corrected timing for pixel 0 in tile drawing so it is consistant with other pixels
;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;/ INCLUDES 
;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

; Include our register and chip definitions file
.include "p24HJ256GP206.inc"

;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;/ DEFINES
;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

; width and height of screen 
.equ NTSC_SCREEN_WIDTH,     240
.equ NTSC_SCREEN_HEIGHT,    192

; controls the number of lines to shift the display vertically, later assign to global variable, so caller can control
.equ NTSC_VERTICAL_OFFSET,  40

; Number of clocks per pixel (12 in the case of NTSC) Modifying this won't really change anything,
; you will need to change code delays to get the correct clocks per pixel change
.equ CLOCKS_PER_PIXEL, 8

; Number of lines we should repeat to reduce the vertical resolution
.equ LINE_REPT_COUNT,   1

; with the current D/A converter range 0 .. 15, 1 = 62mV
.equ NTSC_SYNC,           (0x0F)     ; 0V  
.equ NTSC_BLACK,          (0x4F)     ; 0.3V (about 30-40 IRE)

.equ NTSC_CBURST0,        (0x30) 
.equ NTSC_CBURST1,        (0x31) 
.equ NTSC_CBURST2,        (0x32) 

.equ NTSC_CHROMA_OFF, 	0x000F
.equ NTSC_OVERSCAN_CLR,	0x005F

.equ COLOR_LUMA,    (NTSC_CBURST1 + 0x40)

.equ NTSC_GREEN,    (COLOR_LUMA + 0)        
.equ NTSC_YELLOW,   (COLOR_LUMA + 2)        
.equ NTSC_ORANGE,   (COLOR_LUMA + 4)        
.equ NTSC_RED,      (COLOR_LUMA + 5)        
.equ NTSC_VIOLET,   (COLOR_LUMA + 9)        
.equ NTSC_PURPLE,   (COLOR_LUMA + 11)        
.equ NTSC_BLUE,     (COLOR_LUMA + 14)        

.equ NTSC_GRAY0,    (NTSC_BLACK + 0x10)        
.equ NTSC_GRAY1,    (NTSC_BLACK + 0x20)        
.equ NTSC_GRAY2,    (NTSC_BLACK + 0x30)        
.equ NTSC_GRAY3,    (NTSC_BLACK + 0x40)        
.equ NTSC_GRAY4,    (NTSC_BLACK + 0x50)        
.equ NTSC_GRAY5,    (NTSC_BLACK + 0x60)        
.equ NTSC_GRAY6,    (NTSC_BLACK + 0x70)        
.equ NTSC_GRAY7,    (NTSC_BLACK + 0x80)        
.equ NTSC_GRAY8,    (NTSC_BLACK + 0x90)        
.equ NTSC_GRAY9,    (NTSC_BLACK + 0xA0)        
.equ NTSC_WHITE,    (NTSC_BLACK + 0xB0) 

.equ Sprite_Ptr_Size,    0x04    ; size of sprite pointer


;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;/ EXTERNALS
;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

.extern _g_SpriteList           ; 16-bit pointer to sprite table records
.extern _g_TileSet              ; 16-bit pointer to 8x8 tile bitmaps, allows indirection
.extern _g_MapScroll			; 16-bit pointer to map scrolling struct
;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;/ SRAM SECTIONS
;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

; The following section describes initialized (data) near section. The reason we want
; to place these variables into the near SRAM area is so that ASM codes, that don't use many
; bits to encode data, will be able to access the variables.
.section *,data,near
_g_CurrentLine:		.int 245		        ; Line count 0->261
_g_RasterLine:      .int 0                  ; Virtual Raster count 0->96
ReptLine:	        .int LINE_REPT_COUNT	; Repeated line counter 0->2
CurrVRAMPtr:	    .int 0			        ; Line count 0->525
CurrAttribPtr:	    .int 0			        ; Points to g_AttributeBuf that is defined in the C program as a global
_g_VsyncFlag:       .int 0			        ; Flag to indicate we are in v-sync
ReptAttrib:	        .int 8			        ; Repeat attribute table increment 8 pixel lines

; Un-initialized Data sections
.section *,bss,near
_g_MapScroll_temp: .int 0       ; 16-bit pointer to current position in mapscrolling struct
ScanLine: .space (256+16)	; holds scanline where we render sprites to
Scroll_Scanline: .space 2   ;holds the address of the scrolling info for that scanline

;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;/ GLOBAL EXPORTS
;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

.global __T1Interrupt   ; Declare Timer 1 ISR name global
.global _g_VsyncFlag    ; read only, Flag to indicate we are in v-sync
.global _g_CurrentLine  ; read only, current physical line 0...261
.global _g_RasterLine   ; Virtual Raster count 0->96
;////////////////////////////
;/ Setup the sprite - 5 cycles
;////////////////////////////
.macro Config_Sprite
      ;set up scanline for all sprites (done once per scanline)
        mov _g_SpriteList, W2; (1) pointer for sprite data        
        mov _g_RasterLine, W5; (1) find out which scanline we are using
        add W5, #16, W5 ; (1) add this so we can move sprite offscreen on the Y axis
        mov #0, W4     ; (1) make sure that upper 8 bits are clear for sprite Y value
        mov #0, W10    ; (1) used for compare for sprite
.endm
;////////////////////////////
;/ Drawing 1st sprite 31 cycles (29 and 2 for goto)
;////////////////////////////
.macro Draw_First_Sprite
        ;and #0xFF, W5    ;(1) ***New*** Clear out upper 8 bits
		mov #0, W4    ;(1) ***New*** Clear out upper 8 bits
		mov.b [W2+1], W4 ;(1) get Y value for sprite
        sub W5, W4, W4   ;(1) scanline - sprite Y (should set overflow)
		BRA n, 1f ;(1/2) if sprite Y is greater than current scanline, then don't draw sprite
       ;Sprite Y is less than or equal to Y scanline (possible sprite drawing)
		;and #0xFF, W4;debug
        cp W4, #15          ;(1) if it greater than 15, then the sprite is not on this scanline        
        BRA GT, 2f  ;(1/2) if it is greater than 15, sprite is not on this scanline
       ; - find the correct sprite scanline value - 6 cycles
        sl W4, #4, W4  ;(1) Multiply by 16 since each sprite line is 16 pixels
        mov #ScanLine, W3 ;(1) get adddress to sprite scanline
		and #0xFF, W1     ;(1) ***New*** Clear out upper 8 bits
        mov.b [W2+0], W1 ;(1) get x value for sprite
        add W3, W1, W3    ;(1) add x value to sprite scanline 
	    mov [W2+2], W1      ;(1) sprite graphic address
        add W1, W4, W1     ;(1) add scanline offset to get correct sprite line
        ;unrolled loop for drawing sprite -  16 cycles
;;      This will place sprite without regards to transparency
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1], [W3]   ;(1)
        goto 3f            ;(2)
0:
1:
;-- Time to kill if we aren't drawing the first sprite
;No_sprite_1c0:  ;No sprite, 1st check (Sprite Y greater/lower than current scanline)
        repeat #(31-5-5)    ;(1)
        nop               ;(1+n)
        mov #0, W4        ;(1)  clear out upper 8 bits of W4 register
		goto 3f ;(2)
2:
;No_sprite_2c0:  ;No sprite, 2nd check (Passed 1st check, but sprite not being drawn on this scanline)
        repeat #(31-7-4)    ;(1)
        nop               ;(1+n)
		goto 3f ;(2)
3:
.endm
;////////////////////////////
;/ Sprite Drawing Macro - 63 Cycles
;////////////////////////////
.macro Draw_Sprite
        add #Sprite_Ptr_Size, W2        ;(1) advance sprite pointer to get info for this sprite
        mov.b [W2+1], W4 ;(1) get Y value for sprite
		;mov #0, W4; debug
        sub W5, W4, W4   ;(1) scanline - sprite Y (should set overflow)
		BRA n, 1f ;(1/2) if sprite Y is greater than current scanline, then don't draw sprite
       ;Sprite Y is less than or equal to Y scanline (possible sprite drawing)
        cp W4, #15          ;(1) if it greater than 15, then the sprite is not on this scanline        
        BRA GT, 2f  ;(1/2) if it is greater than 15, sprite is not on this scanline
       ; - find the correct sprite scanline value - 6 cycles
        sl W4, #4, W4  ;(1) Multiply by 16 since each sprite line is 16 pixels
        mov #ScanLine, W3 ;(1) get adddress to sprite scanline
		;and #0xFF, W1     ;(1) ***New*** Clear out upper 8 bits
		mov #0, W1     ;(1) ***New*** Clear out upper 8 bits
        mov.b [W2+0], W1 ;(1) get x value for sprite
        add W3, W1, W3    ;(1) add x value to sprite scanline 
	    mov [W2+2], W1      ;(1) sprite graphic address
        add W1, W4, W1     ;(1) add scanline offset to get correct sprite line
        ;unrolled loop for drawing sprite - 50 cycles (including goto)
        ;pixel 0
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+0]  ;(1)
        ;pixel 1
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+1]  ;(1)
        ;pixel 2
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+2]  ;(1)
        ;pixel 3
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+3]  ;(1)
        ;pixel 4
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+4]  ;(1)
        ;pixel 5
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+5]  ;(1)
        ;pixel 6
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+6]  ;(1)
        ;pixel 7
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+7]  ;(1)
        ;pixel 8
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+8]  ;(1)
        ;pixel 9
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+9]  ;(1)
        ;pixel 10
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+10]  ;(1)
        ;pixel 11
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+11]  ;(1)
        ;pixel 12
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+12]  ;(1)
        ;pixel 13
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+13]  ;(1)
        ;pixel 14
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+14]  ;(1)
        ;pixel 15
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+15]  ;(1)
0: ;first skip
        goto 3f     ;(2) 
;-- Time to kill if we aren't drawing the second or 4th sprite
1:  ;No sprite, 1st check (Sprite Y greater/lower than current scanline)
        repeat #(64-5-5)    ;(1)
        nop               ;(1+n)
        mov #0, W4        ;(1)  clear out upper 8 bits of W4 register
		goto 3f ;(2)
2:  ;No sprite, 2nd check (Passed 1st check, but sprite not being drawn on this scanline)
        repeat #(64-7-4)    ;(1)
        nop               ;(1+n)
		goto 3f ;(2)
3:
.endm

;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;/ ASM FUNCTIONS
;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

; The following are code sections
.text           

__T1Interrupt:

		; Our instruction clock is running at 12*NTSC dot clock
		; so every 12 clock cycles is a normal 227 pixels
		disi #0x3FFF					; (1) Disable interupts for up to 10000 cycles

		; Save off variables we will use
		push.s						; (1) Push status registers
		push SR
		push W0
        push W1
		push W2
        push W3
		push W4						; (1) Temp variable
		push W5
        push W6
        push W7
        push W8
		push W9
		push W10
		push RCOUNT					; (1) Make sure we are pushing Rcount since we are using repeat

		; Increment which line we are on
		inc _g_CurrentLine				; (1)
		mov _g_CurrentLine, W0				; (1) Set our line variable into W0

		; Test if we are done with this frame (262 lines)
		mov #261, W2					; (1)
		cp W0, W2					; (1) See if we are at line 262 = 0
		bra nz, DONE_LINE_TEST				; (1/2) if(_g_CurrentLine != 262) jump
		setm _g_CurrentLine				; (1) Reset Line counter set to 0xFFFF
DONE_LINE_TEST: 

		; Total = 10

        ; The following section of code determines what kind of line we should be 
        ; drawing to the screen. Is it a Vsync Line, Black Top/Bottom, or an
        ; Active Video line. All of this is determined by the _g_CurrentLine counter
        ; variable

        ; if(_g_CurrentLine < VSYNC_LINE_COUNT)
		mov #5, W2					; (1)
		cp W0, W2					; (1) Compare to 5
		bra gt, NOT_VSYNC_LINE				; (1/2) if(Line > 5) jump

		; Vsync line (18)
		repeat	#11					; (1)
		nop						; (1+n)
		goto DRAW_VSYNC_LINE				; (2) draw a black line to the screen

NOT_VSYNC_LINE:

		; Test to see if this is a top blanking line
		mov #NTSC_VERTICAL_OFFSET, W2			; (1)
		cp W0, W2					; (1) Compare to NTSC_VERTICAL_OFFSET
		bra ge, NOT_TOP_OVERSCAN_LINE			; (1/2) if(Line >= NTSC_VERTICAL_OFFSET) jump

		; Blank Line Top (18)
		nop						; (1)
        nop							; (1)
        clr _g_RasterLine                          		; (1) Clear out our raster line count
		nop ;mov _g_CurrentVRAMBuff, W1			; (1) Copy over CurrVRAMPtr = _g_CurrentVRAMBuff
		nop ;mov W1, CurrVRAMPtr				; (1) copy
		nop ;mov _g_AttribTable, W1				; (1) Copy over ATTRIBTABLE = g_AttributeTable
		nop ;mov W1, CurrAttribPtr				; (1) copy
		mov #LINE_REPT_COUNT, W1			; (1) Reset repeat line back to count
		mov W1, ReptLine							; (1)
		goto DRAW_OVERSCAN_LINE				; (2) draw a black line to the screen

NOT_TOP_OVERSCAN_LINE:

		; Test to see if this is a bottom blanking line
		mov #((NTSC_SCREEN_HEIGHT*LINE_REPT_COUNT)+NTSC_VERTICAL_OFFSET), W2	; (1)
		cp W0, W2					; (1) Compare to 245
		bra lt, NOT_BOTTOM_OVERSCAN_LINE		; (1/2) if(Line < 245) jump

		; Blank Line Bottom (18)
		repeat	#2					; (1)
		nop						; (1+n)
		setm _g_VsyncFlag				; (1) set the vsync flag 
		goto DRAW_OVERSCAN_LINE				; (2) draw a black line to the screen

NOT_BOTTOM_OVERSCAN_LINE:

		; This must be an active line (18)
		clr _g_VsyncFlag				; (1) g_VsyncFlag = 0
		nop						; (1) delay
		nop						; (1) delay
		nop						; (1) delay
		goto DRAW_ACTIVE_LINE				; (2) draw an active line to the screen



; ----------------- VSYNC LINE -------------------------
DRAW_VSYNC_LINE:

		; This draws a vsync line with h-sync pulse inverted for serration
		; H-sync serration (4.7us) 236
		mov #NTSC_BLACK,W0 	                        ; (1) turn hsync on
		mov W0, LATB					; (1)
		repeat #232					; (1)
		nop						; (1+n)
		mov #NTSC_SYNC, W0	                        ; (1)
		mov W0, LATB					; (1)

		; Done with this completely
		goto FINISHED_NTSC		; Dont need to count because interrupts will keep us locked



; ----------------- OVERSCAN LINE -------------------------
DRAW_OVERSCAN_LINE:

		; This draws a overscan line with h-sync (used for blanking lines)
		; H-sync (4.7uS) = 202
		mov #NTSC_SYNC, W0	                        ; (1) turn hsync on
		mov W0, LATB					; (1)
;clear sprite scanline, we do this because if the sprite scrolls off the bottom of the screen, then it will 
;draw on the top scanline otherwise
	    mov #ScanLine, W3 ;(1) get adddress to sprite scanline
        add W3, #16, W3   ;(1) advance to the visable part of the scanline
        mov #0, W1        ;(1) value to clear scanline with
        repeat #120     ;(1)
        mov W1, [W3++]  ;(1+n) clear scanline 
		;/Draw first sprite, needed here if the sprite is on the first scaneline
		Config_Sprite ;(5)
		mov #15, W5; (1) set as first scanline always (Draw_First_Sprite will add 1 to make it 16)
	    Draw_First_Sprite ;(31)

		repeat #(198-125-37)		; (1)
		nop						; (1+n)

		; Pre-burst (.6uS) = 26 clocks
		mov #NTSC_BLACK, W0	                        ; (1) turn hsync off black with no color signal
		mov W0, LATB					; (1)
		repeat #22					; (1)
		nop						; (1+n)


		; Color burst (10 NTSC clocks) = 120 clocks
		mov #NTSC_CBURST1, W0				; (1) turn hsync color burst on at phase 0
		mov W0, LATB					; (1)
	    Draw_Sprite     ;(63) Sprite drawing function
		repeat #(120-4-63)				; (1)
		nop						; (1+n)


		; Post-burst (.6uS) = 26 clocks
		mov #NTSC_BLACK, W0	                        ; (1) turn hsync off black with no color signal
		mov W0, LATB					; (1)
        mov #_g_MapScroll_temp, W0       ;(1) get address for temp MapScroll struct
        mov _g_MapScroll, W1  ;(1) get address for MapScroll struct
        mov W1, [W0]          ;(1) copy address for MapScroll struct
		repeat #(26-4-3)					; (1)
		nop						; (1+n)

		; Overscan color
		mov #NTSC_OVERSCAN_CLR, W0			; (1) turn hsync off black with a color signal
		mov W0, LATB					; (1)
        Finish_Scanline_0:
		; Done with this completely
		goto FINISHED_NTSC				; Dont need to count because interrupts will keep us locked




; ----------------- ACTIVE LINE -------------------------
DRAW_ACTIVE_LINE:

;---------------------------------------------------------------
;Hsync On - 202 Clocks - Sprites 3, 4, and 5
;    Clocks: Hsync - 5 clocks, sprite setup - 7 clocks, 3 sprites - 189 clocks, goto - 1 clocks (we get an extra from not branching)
;---------------------------------------------------------------
		; ------- HSYNC ON ---------
		; 43 * X.XuS = clock cycles
		; H-sync (4.7uS) = 202
		mov #NTSC_SYNC, W0 	            ; (1) turn hsync on
		mov W0, LATB					; (1)
		Config_Sprite ;(5)
        add #Sprite_Ptr_Size, W2        ;(1) advance sprite pointer since we are starting on the 3rd sprite
        ; 3 sprites, 189 clocks
		Draw_Sprite ;(63)
		Draw_Sprite ;(63)
		Draw_Sprite ;(63)
        ;5 nops
        nop                             ; (1)
        nop                             ; (1)
        nop                             ; (1)
        nop                             ; (1)
        nop                             ; (1)
;---------------------------------------------------------------
;Pre-burst - 26 clocks
;---------------------------------------------------------------
        PRE_BURST:
		; Pre-burst (.6uS) = 26 clocks
		mov #NTSC_BLACK, W0	                        ; (1) turn hsync off black with no color signal
		mov W0, LATB					; (1)
		repeat #22					; (1)
		nop						; (1+n)
;---------------------------------------------------------------
;Color Burst - 119 clocks - Sprite 6
;---------------------------------------------------------------
        ; 2.5uS - 108 clock cycles
		; Color burst (10 NTSC clocks) = 80 clocks (originally 120 clocks)
		mov #NTSC_CBURST1, W0 		        ; (1) turn hsync color burst on at phase 0
		mov W0, LATB				; (1)
	    Draw_Sprite     ;(63) Sprite drawing function
	    repeat #(120-63-4-1)				; (1)
		nop					; (1+n)

;---------------------------------------------------------------
;Post Burst - 65 Clocks - Sprite 7
;---------------------------------------------------------------
        ; Post-burst 1.6uS - 69
        POST_BURST:
		mov #NTSC_BLACK, W0	                        ; (1) turn hsync off black with no color signal
		mov W0, LATB			; (1)
	    Draw_Sprite     ;(63) Sprite drawing function
;---------------------------------------------------------------
;Overscan - 120 Clocks (115 before we start drawing tiles) - Sprite 8 
;---------------------------------------------------------------
		OVER_SCAN:
		;Overscan 
		; FROM HERE ON it is imparative that we stay in sync with the color clock signal
		; (i.e. we don't change any pixels unless it is an integer multiple of 8)
		; If we don't do this then color changes will not be correct.
		; Post burst
        mov #(NTSC_OVERSCAN_CLR | NTSC_CHROMA_OFF), W0 ; (1) turn hsync off black with no color signal
		mov W0, LATB					; (1)
	    Draw_Sprite     ;(63) Sprite drawing function
		;make sure we are in sync with color clock
		repeat #(3-2)
		nop
PIXEL_LOOP:     
;-------------------------------------------------------------------------------------------------
;256 = 7 clocks a pixel (=1792 clocks)
;240 = 8 clocks a pixel
;192 = 10 clocks a pixel
;160 = 12 clocks a pixel
        
; W0 - color to write to screen, tile address from tile map
; W1 - pointer to TileSet
; W2 - counts how many pixels we have left to draw in the scanline, get scanline to find correct tile address
; W3 - counter for how many pixels we have left to draw for tile
; W4 - horizontal offset for tile
; W5 - address for current tile map
;----- Set up scanline for tiles (33 cycles) (48 total)(50 before we are even with color clock)
      Setup_scanline:
		mov _g_TileSet, W1			; (1) Copy over CurrVRAMPtr = _g_CurrentVRAMBuff
        mov #0, W3; (1)
		mov _g_MapScroll_temp, W4  ; (1) get Y scroll value
		mov.b [W4], W4          ; (1) get value for Y
		mov _g_RasterLine, W2 ; (1) grab current scanline
        add W2, W4, W4        ; (1) add current scanline to scroll
		mov W4, W2            ; (1) copy register
        and #0x07, W4         ;(1) mask it off so we can use it for tiles
        sl W4, #3, W4            ;(1) multiply by 8 for offset
        add W1, W4, W1            ;(1) set new offset for tile
		and #0xFF, W2         ; (1) make sure we don't go past tile map
		asr W2, #3, W2            ;(1) divide by 8 to get the tile row		
		sl  W2, #5, W2        ;(1) multiply by 32 (number of horizontal tiles)
		mov W2, W5               ;(1) move back to original register
        mov _g_MapScroll_temp, W0 ;(1) grab the address from the tile map
		add W0, #4, W0        ;(1) add 4 to get to map address
		mov [W0], W0          ;(1) read the address of tilemap
		add W5, W0, W5        ;(1) get address for tile map
		mov #0, W0 ;(1) clear upper 8 bits of W6 register for tile byte (make sure it is only an 8 bit value)
        mov #ScanLine, W4 ;(1) get adddress to sprite scanline
        add #16, W4           ; (1) add 16 since that is the first visible pixel
		mov #0, W6          ;(1) for compare (if sprite pixel is zero then load graphics tile)
	    mov #(240-8), W2; (1) pixel counter for number of pixels to draw
		;setup first tile
		mov _g_MapScroll_temp, W8  ; (1) get scrolling address
		add W8, #1, W8        ;(1) get to X value
        mov.b [W8], W8        ;(1) get X value
		and #0xFF, W8         ;(1) mask W8 to ensure we have an 8-bit value
        mov W8, W9            ;(1) grab value of X Scroll
        asr W8, #3, W8        ;(1) mask since we only only scroll 8 for chunky scrolling
        add W8, W5, W5        ;(1) add map X scroll 
		mov.b [W5++], W0 ; (1) get next tile we are rendering from tile map        
		sl W0, #6, W10;   (1) multiply by 64 (each tile is 64 bytes) 
		add W10, W1, W10  ;(1) combine tile graphics offset to tile image (for getting next tile)
;--First draw a partial 1st tile, then draw 29 tiles, then draw a partial last tile (15 clocks)
        mov W10, W3      ;(1)copy next pointer value for tile
		mov #8, W7       ;(1) value to subract from
        and #7, W9       ;(1) we only need first 3 bits
        cpsne W9, W6     ;(1) if it is not 0, then subtract 8 to draw all tiles
        sub W2, #8, W2   ;(1) we are drawing 1 less tile, subtract
        sub W7, W9, W7   ;(1) get how many pixels we will draw (the other we chop off)
        add W9, W3, W3   ;(1) pixels to chop off
        cpsne W9, W6     ;(1) if it is not 0, then subtract 8 to draw all tiles
        mov #8, W9   ;(1) draw a full tile for last tile
        ;ready 2nd tile
        add #1, W8      ;(1) advance to next tile
        and W8, #31, W8 ;(1) check to see if we need to wrap back around
        cpsne W6,W8     ;(1/2) if we need to wrap, subtract 32 
        sub #32, W5     ;(1) if we do wrap, start at beginning of map
		mov.b [W5++], W0 ; (1) get next tile we are rendering from tile map
		sl W0, #6, W10;   (1) multiply by 64 (each tile is 64 bytes) 
;k---- Place rendering engine here (1920 clocks - 8 clocks a pixel - 240 pixels)
Draw_First_Tile: ;Draw chunk of first tile, erasing the scanline buffer for this tile is done at the end
        mov.b [W4++], W0 ;(1) copy pixel from pixel buffer
        cpsne W0, W6 ;(1/2) if pixel is not zero, then skip instruction to read tile pixel
        mov.b [W3],W0  ;(1) get pixel by taking pointer to tileset, adding pixel number for tile, adding tile from time map
        add W3, #1, W3 ;(1) advance pixel pointer
        mov W0, LATB ; (1) render pixel signal to screen
        sub W7, #1, W7  ; (1) count off remaining of this tile
		BRA NZ, Draw_First_Tile ;(1/2) if sprite Y is greater than current scanline, then don't draw sprite
		add W10, W1, W10  ;(1) combine tile graphics offset to tile image (for getting next tile)        

Draw_Pixel:
		; unroll loop for a full tile (8 pixels)
;pixel 0--------------------- (note: the sprite pixel line check throws off the pixel clock pushing it 2 clocks down)
        mov W10, W3      ;(1)copy next pointer value for tile
		mov.b [W4++], W0 ;(1) copy pixel from pixel buffer--
        cpsne W0, W6 ;(1/2) if pixel is not zero, then skip instruction to read tile pixel--
        mov.b [W3],W0  ;(1) get pixel by taking pointer to tileset, adding pixel number for tile, adding tile from time map
        mov W0, LATB ; (1) render pixel signal to screen
        mov.b W6, [W4-1]; ;(1) erase scanline value for pixel 0
        add #1, W8      ;(1) advance to next tile
        and W8, #31, W8 ;(1) check to see if we need to wrap back around
;pixel 1---------------------
		nop;
        mov.b [W4++], W0 ;(1) copy pixel from pixel buffer
        cpsne W0, W6 ;(1/2) if pixel is not zero, then skip instruction to read tile pixel
        mov.b [W3+1],W0  ;(1) get pixel by taking pointer to tileset, adding pixel number for tile, adding tile from time map
        mov W0, LATB ; (1) render pixel signal to screen
        mov.b W6, [W4-1]; ;(1) erase scanline value for pixel 1
        cpsne W6,W8     ;(1/2) if we need to wrap, subtract 32 
        sub #32, W5     ;(1) if we do wrap, start at beginning of map
;pixel 2---------------------
        mov.b [W4++], W0 ;(1) copy pixel from pixel buffer
        nop ;(1)
        cpsne W0, W6 ;(1/2) if pixel is not zero, then skip instruction to read tile pixel
        mov.b [W3+2],W0  ;(1) get pixel by taking pointer to tileset, adding pixel number for tile, adding tile from time map
        mov W0, LATB ; (1) render pixel signal to screen
        mov.b W6, [W4-1]; ;(1) erase scanline value for pixel 2
		mov.b [W5], W0 ; (1) get next tile we are rendering from tile map
		sl W0, #6, W10;   (1) multiply by 64 (each tile is 64 bytes) 
;pixel 3---------------------
		add W10, W1, W10  ;(1) combine tile graphics offset to tile image (for getting next tile)        
		mov.b [W4++], W0 ;(1) copy pixel from pixel buffer
		cpsne W0, W6 ;(1/2) if pixel is not zero, then skip instruction to read tile pixel
        mov.b [W3+3],W0  ;(1) get pixel by taking pointer to tileset, adding pixel number for tile, adding tile from time map
        mov W0, LATB ; (1) render pixel signal to screen
        mov.b W6, [W4-1]; ;(1) erase scanline value for pixel 3
		add #1, W5     ;(1) next tile in tile map
        nop ;(1)
;pixel 4---------------------
        mov.b [W4++], W0 ;(1) copy pixel from pixel buffer
        nop ;(1)
        cpsne W0, W6 ;(1/2) if pixel is not zero, then skip instruction to read tile pixel
        mov.b [W3+4],W0  ;(1) get pixel by taking pointer to tileset, adding pixel number for tile, adding tile from time map
        mov W0, LATB ; (1) render pixel signal to screen
        mov.b W6, [W4-1]; ;(1) erase scanline value for pixel 4
		nop ;(1)
        nop ;(1)
;pixel 5---------------------
        mov.b [W4++], W0 ;(1) copy pixel from pixel buffer
        nop ;(1)
        cpsne W0, W6 ;(1/2) if pixel is not zero, then skip instruction to read tile pixel
        mov.b [W3+5],W0  ;(1) get pixel by taking pointer to tileset, adding pixel number for tile, adding tile from time map
        mov W0, LATB ; (1) render pixel signal to screen
        mov.b W6, [W4-1]; ;(1) erase scanline value for pixel 5
		nop ;(1)
        nop ;(1)
;pixel 6---------------------
        mov.b [W4++], W0 ;(1) copy pixel from pixel buffer
        nop ;(1)
        cpsne W0, W6 ;(1/2) if pixel is not zero, then skip instruction to read tile pixel
        mov.b [W3+6],W0  ;(1) get pixel by taking pointer to tileset, adding pixel number for tile, adding tile from time map
        mov W0, LATB ; (1) render pixel signal to screen
        mov.b W6, [W4-1]; ;(1) erase scanline value for pixel 6
		nop ;(1)
        nop ;(1)
;pixel 7---------------------
        mov.b [W4++], W0 ;(1) copy pixel from pixel buffer
        mov.b W6, [W4-1]; ;(1) erase scanline value for pixel 7
        cpsne W0, W6 ;(1/2) if pixel is not zero, then skip instruction to read tile pixel
        mov.b [W3+7],W0  ;(1) get pixel by taking pointer to tileset, adding pixel number for tile, adding tile from time map
		mov W0, LATB ; (1) render pixel signal to screen
        sub W2, #8, W2;  (1) - subtract 8 from amount of pixels left to draw
		bra nz, Draw_Pixel	; (1/2) check to see if we draw more pixels
		mov W10, W3      ;(1)copy next pointer value for last tile

;Draw the last tile
Draw_Last_Tile: ;Draw chunk of first tile, erasing the scanline buffer for this tile is done at the end
        mov.b [W4++], W0 ;(1) copy pixel from pixel buffer
        cpsne W0, W6 ;(1/2) if pixel is not zero, then skip instruction to read tile pixel
        mov.b [W3],W0  ;(1) get pixel by taking pointer to tileset, adding pixel number for tile, adding tile from time map
        add W3, #1, W3 ;(1) advance pixel pointer
        mov W0, LATB ; (1) render pixel signal to screen
        sub W9, #1, W9  ; (1) count off remaining of this tile
		BRA NZ, Draw_Last_Tile ;(1/2) if sprite Y is greater than current scanline, then don't draw sprite
        nop ;(1)
Debug_Skip:
;---------------------------
; Draw righthanded overscan
;---------------------------
		; Now set PORTB to overscan
		mov #(NTSC_OVERSCAN_CLR | NTSC_CHROMA_OFF), W0 ; (1) turn hsync off black with no color signal
		mov _g_MapScroll_temp, W3  ; (1) get scrolling address
		add W3, #2, W4        ;(1) get address to last line value
		mov W0, LATB			; (1) include two cycle delay
		mov.b [W4], W4          ; (1) get value for Last line
        and #0xFF, W4           ; (1) clear out upper 8 bits
        mov _g_RasterLine, W8 ; (1) grab current scanline        
        mov #0, W0 ;(1)  default value to add to MapScroll struct 
        cpsne W4, W8 ;(1/2) if pixel is not zero, then skip instruction to read tile pixel        
        ;mov #3, W0; (1) value to add to get to next MapScroll struct 
        mov #6, W0; (1) value to add to get to next MapScroll struct 
        mov #_g_MapScroll_temp, W4 ;(1) get address for MapScroll struct
        add W0, W3,[W4]            ;(1) get new address of MapScroll struct
		; Erase the 1st 8 and last 8 pixels because first and last tile omit this
        ;mov #0xFFFF, W6   ;debug ------
	    mov #ScanLine, W4 ;(1) get adddress to sprite scanline
        add W4, #16, W4   ;(1) advance to the visable part of the scanline
        mov W6, [W4++]  ;(1) clear 2 scanline pixels
        mov W6, [W4++]  ;(1) clear 2 scanline pixels
        mov W6, [W4++]  ;(1) clear 2 scanline pixels
        mov W6, [W4++]  ;(1) clear 2 scanline pixels
        mov #(240-16), W0 ;(1) get how many we are going to advance
		add W4, W0, W4  ;(1) get new location
        mov W6, [W4++]  ;(1) clear 2 scanline pixels
        mov W6, [W4++]  ;(1) clear 2 scanline pixels
        mov W6, [W4++]  ;(1) clear 2 scanline pixels
        mov W6, [W4++]  ;(1) clear 2 scanline pixels

;-------------------------------------------------------------------------------------------------
; Draw 1st and 2nd sprites on scanline
;-------------------------------------------------------------------------------------------------
      
		Config_Sprite ;(5) ;set up scanline for all sprite
        add W8, #1, W8       ; (1) add one since we are drawing this for the next scanline
	    Draw_First_Sprite ;(31)
	    Draw_Sprite     ;(63) Sprite drawing function
        Finish_Scanline:
;-------------------------------------------------------------------------------------------------
        inc _g_RasterLine                       ; (1) Increment our raster line count
		; Done with this completely
FINISHED_NTSC:

        bclr IFS0, #T1IF           		; Clear the Timer1 Interrupt flag Status
                                   		; bit.

		pop RCOUNT			; (1) Make sure we are pushing Rcount since we are using repeat
		pop W10
		pop W9
        pop W8
        pop W7
        pop W6
		pop W5
		pop W4                   	; Retrieve context POP-ping from Stack
        pop W3
        pop W2
		pop W1
		pop W0                   	; Retrieve context POP-ping from Stack
		pop SR
		pop.s				; (1) Push status registers

		disi #0x0			; Re-enable interrupts

        RETFIE                     		;Return from Interrupt Service routine




;--------End of All Code Sections ---------------------------------------------

.end                               ;End of program code in this file
        